# # 创建得分Score类

import pygame
from pygame.sprite import Group
from ship import Ship


class Scoreboard():
    """显示得分信息的类"""
    def __init__(self, ai_settings, screen, game_stats):
        """初始化显示得分涉及的属性"""
        self.screen = screen
        self.screen_rect = screen.get_rect()
        self.ai_settings = ai_settings
        self.game_stats = game_stats

        # 显示得分信息时使用的字体设置
        self.text_color = (30, 30, 30)
        self.font = pygame.font.SysFont(None, 48)
        self.prep_images()

    def prep_images(self):
        """准备包含 最高得分、当前得分、游戏等级、飞船 的图像"""
        # 当前得分图像
        self.prep_score()

        # 最高得分图像
        self.prep_high_score()

        # 等级图像
        self.prep_level()

        # 飞船组图像
        self.prep_ships()

    def prep_score(self):
        """将当前得分转换为一幅渲染的图像"""
        round_score = round(self.game_stats.score, -1)  # 使得分为10的倍数  -1----10的倍数   -2--100的倍数  -3--1000的倍数
        score_str = "score:" + "{:,}".format(round_score)
        self.score_image = self.font.render(score_str, True, self.text_color,
                                                self.ai_settings.bg_color)

        # 将当前得分放在屏幕右上角
        self.score_image_rect = self.score_image.get_rect()
        self.score_image_rect.right = self.screen_rect.right - 20
        self.score_image_rect.top = 20

    def prep_high_score(self):
        """将最高得分转换为一幅渲染的图像"""
        high_round_score = round(self.game_stats.high_score, -1)  # 使得分为10的倍数  -1----10的倍数   -2--100的倍数  -3--1000的倍数
        high_score_str = "Mscore:" + "{:,}".format(high_round_score)
        self.high_score_image = self.font.render(high_score_str, True, self.text_color,
                                            self.ai_settings.bg_color)

        # 将最高得分放在顶部屏幕中央
        self.high_score_image_rect = self.high_score_image.get_rect()
        self.high_score_image_rect.centerx = self.screen_rect.centerx
        self.high_score_image_rect.top = self.score_image_rect.top

    def prep_level(self):
        """将等级转换为一幅渲染的图像"""

        self.level_image = self.font.render(("lev:" + str(self.game_stats.level)), True,
                                            self.text_color, self.ai_settings.bg_color)

        # 将等级放在等分下面
        self.level_image_rect = self.level_image.get_rect()
        self.level_image_rect.right = self.score_image_rect.right
        self.level_image_rect.top = self.score_image_rect.bottom + 10

    def prep_ships(self):
        """显示还剩下多少艘飞船"""
        self.ships = Group()
        for ship_number in range(self.game_stats.ships_left):
            ship = Ship(self.ai_settings, self.screen)
            ship.rect.x = 10 + ship_number * ship.rect.width
            ship.rect.top = 10
            self.ships.add(ship)

    def show_score(self):
        """在屏幕上显示当前得分和最高得分"""
        self.screen.blit(self.score_image, self.score_image_rect)
        self.screen.blit(self.high_score_image, self.high_score_image_rect)
        self.screen.blit(self.level_image, self.level_image_rect)
        # 绘制飞船
        self.ships.draw(self.screen)